@@ -1,3 +1,4 @@
+/* -*- mode: C++; c-basic-offset: 2; indent-tabs-mode: nil -*- */
 /*
  *  Main authors:
  *     Christian Schulte <schulte@gecode.org>
@@ -6,16 +7,31 @@
  *     Christian Schulte, 2001
  *
  *  Last modified:
- *     $Date: 2006-03-09 11:12:26 +0100 (Thu, 09 Mar 2006) $ by $Author: zayenz $
- *     $Revision: 3063 $
+ *     $Date: 2007-11-30 13:58:34 +0100 (Fri, 30 Nov 2007) $ by $Author: tack $
+ *     $Revision: 5524 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
  *     http://www.gecode.org
  *
- *  See the file "LICENSE" for information on usage and
- *  redistribution of this file, and for a
- *     DISCLAIMER OF ALL WARRANTIES.
+ *  Permission is hereby granted, free of charge, to any person obtaining
+ *  a copy of this software and associated documentation files (the
+ *  "Software"), to deal in the Software without restriction, including
+ *  without limitation the rights to use, copy, modify, merge, publish,
+ *  distribute, sublicense, and/or sell copies of the Software, and to
+ *  permit persons to whom the Software is furnished to do so, subject to
+ *  the following conditions:
+ *
+ *  The above copyright notice and this permission notice shall be
+ *  included in all copies or substantial portions of the Software.
+ *
+ *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ *  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+ *  LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+ *  OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+ *  WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  */
 
@@ -26,7 +42,7 @@
  *
  * Standard benchmark for solving linear equations.
  *
- * \ingroup Example
+ * \ingroup ExProblem
  *
  */
 class Eq20 : public Example {
@@ -67,10 +83,10 @@ public:
     // Post equation constraints
     for (int i = e_n; i--; ) {
       IntArgs c(7,eqs[i][1],eqs[i][2],eqs[i][3],eqs[i][4],
-		eqs[i][5],eqs[i][6],eqs[i][7]);
-      linear(this, c, x, IRT_EQ, eqs[i][0], opt.icl);
+                eqs[i][5],eqs[i][6],eqs[i][7]);
+      linear(this, c, x, IRT_EQ, eqs[i][0], opt.icl());
     }
-    branch(this, x, BVAR_NONE, BVAL_MIN);
+    branch(this, x, INT_VAR_NONE, INT_VAL_MIN);
   }
 
   /// Constructor for cloning \a s
@@ -84,10 +100,8 @@ public:
   }
   /// Print solution
   virtual void
-  print(void) {
-    std::cout << "\tx[] = {";
-    for (int i = 0; i < x_n; i++)
-      std::cout << x[i] << ((i<x_n-1)?",":"};\n");
+  print(std::ostream& os) {
+    os << "\tx[] = " << x << std::endl;
   }
 
 };
@@ -96,12 +110,12 @@ public:
  *  \relates Eq20
  */
 int
-main(int argc, char** argv) {
+main(int argc, char* argv[]) {
   Options opt("Eq20");
-  opt.solutions  = 0;
-  opt.iterations = 1000;
+  opt.solutions(0);
+  opt.iterations(1000);
   opt.parse(argc,argv);
-  Example::run<Eq20,DFS>(opt);
+  Example::run<Eq20,DFS,Options>(opt);
   return 0;
 }
 
