@@ -1,3 +1,4 @@
+/* -*- mode: C++; c-basic-offset: 2; indent-tabs-mode: nil -*- */
 /*
  *  Main authors:
  *     Christian Schulte <schulte@gecode.org>
@@ -6,16 +7,31 @@
  *     Christian Schulte, 2004
  *
  *  Last modified:
- *     $Date: 2006-08-04 16:06:52 +0200 (Fri, 04 Aug 2006) $ by $Author: schulte $
- *     $Revision: 3517 $
+ *     $Date: 2007-11-30 13:58:34 +0100 (Fri, 30 Nov 2007) $ by $Author: tack $
+ *     $Revision: 5524 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
  *     http://www.gecode.org
  *
- *  See the file "LICENSE" for information on usage and
- *  redistribution of this file, and for a
- *     DISCLAIMER OF ALL WARRANTIES.
+ *  Permission is hereby granted, free of charge, to any person obtaining
+ *  a copy of this software and associated documentation files (the
+ *  "Software"), to deal in the Software without restriction, including
+ *  without limitation the rights to use, copy, modify, merge, publish,
+ *  distribute, sublicense, and/or sell copies of the Software, and to
+ *  permit persons to whom the Software is furnished to do so, subject to
+ *  the following conditions:
+ *
+ *  The above copyright notice and this permission notice shall be
+ *  included in all copies or substantial portions of the Software.
+ *
+ *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ *  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+ *  LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+ *  OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+ *  WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  */
 
@@ -44,7 +60,7 @@ public:
 };
 
 /// First example graph: edges
-static const int g1_e[] = {
+const int g1_e[] = {
   160,184,  192,199,  0,129,  20,80,  8,29,  93,171,
   101,165,  124,193,  2,100,  66,173,  151,191,  164,187,
   3,130,  118,176,  121,184,  25,106,  159,193,  121,123,
@@ -96,7 +112,7 @@ static const int g1_e[] = {
   142,147,  33,165,  86,198,  146,149,  152,187,  44,62,
   48,175,  56,150,  63,161,  71,164,  17,171,  19,66,       -1,-1};
 /// First example graph: cliques
-static const int g1_c[] = {
+const int g1_c[] = {
   30,  6,11,14,25,40,42,48,53,61,76,80,87,89,92,108,115,120,131,132,137,145,159,162,163,164,168,172,173,176,182,
   30,  3,15,16,31,34,35,37,38,49,58,67,78,86,91,100,110,114,123,129,132,133,140,143,154,167,168,174,175,193,197,
   25,  3,10,33,38,43,45,48,51,65,66,82,88,90,93,94,103,107,128,131,141,152,155,168,185,199,
@@ -167,10 +183,10 @@ static const int g1_c[] = {
   -1};
 
 /// First example graph
-static const GraphColorSpec g1(200, g1_e, g1_c);
+const GraphColorSpec g1(200, g1_e, g1_c);
 
 /// Second example graph: edges
-static const int g2_e[] = {
+const int g2_e[] = {
   63,97,  67,85,  18,180,  2,143,  55,156,  28,105,
   37,196,  26,33,  88,199,  175,179,  45,46,  62,70,
   53,58,  49,177,  91,178,  52,129,  147,165,  83,95,
@@ -206,7 +222,7 @@ static const int g2_e[] = {
   38,80,  135,172,  43,156,  148,151,  135,195,  111,184,
   10,14,  97,152,       -1,-1};
 /// Second example graph: cliques
-static const int g2_c[] = {
+const int g2_c[] = {
   30,  10,11,17,20,24,29,33,40,45,50,51,76,77,85,88,101,114,120,122,127,129,136,144,147,148,157,169,180,184,193,
   30,  0,2,14,21,38,40,44,51,72,82,91,99,106,109,119,123,126,136,141,154,157,161,163,165,166,171,175,182,183,196,
   30,  2,17,20,30,35,36,37,39,46,56,61,72,75,77,80,84,85,96,97,100,107,112,123,156,160,163,175,181,182,195,
@@ -280,13 +296,13 @@ static const int g2_c[] = {
   -1};
 
 /// Second example graph
-static const GraphColorSpec g2(200, g2_e, g2_c);
+const GraphColorSpec g2(200, g2_e, g2_c);
 //@}
 
 /**
  * \brief %Example: Clique-based graph coloring
  *
- * \ingroup Example
+ * \ingroup ExProblem
  *
  */
 class GraphColor : public Example {
@@ -297,13 +313,23 @@ private:
   /// Number of colors
   IntVar m;
 public:
+  /// Model variants
+  enum {
+    MODEL_NONE,  ///< No lower bound
+    MODEL_CLIQUE ///< Use maximal clique size as lower bound
+  };
+  /// Branching to use for model
+  enum {
+    BRANCH_DEGREE,      ///< Choose variable with largest degree
+    BRANCH_SIZE,        ///< Choose variable with smallest size
+    BRANCH_SIZE_DEGREE  ///< Choose variable with smallest size/degree
+  };
   /// The actual model
-  GraphColor(const Options& opt)
-    : g(opt.size == 1 ? g2 : g1),
+  GraphColor(const SizeOptions& opt)
+    : g(opt.size() == 1 ? g2 : g1),
       v(this,g.n_v,0,g.n_v),
       m(this,0,g.n_v) {
-    for (int i = g.n_v; i--; )
-      rel(this, v[i], IRT_LQ, m);
+    rel(this, v, IRT_LQ, m);
     for (int i = 0; g.e[i] != -1; i += 2)
       rel(this, v[g.e[i]], IRT_NQ, v[g.e[i+1]]);
 
@@ -314,18 +340,20 @@ public:
       int n = *c;
       IntVarArgs x(n); c++;
       for (int i = n; i--; c++)
-	x[i] = v[*c];
-      distinct(this, x, opt.icl);
-      if (!opt.naive)
-	rel(this, m, IRT_GQ, n);
+        x[i] = v[*c];
+      distinct(this, x, opt.icl());
+      if (opt.model() == MODEL_CLIQUE)
+        rel(this, m, IRT_GQ, n-1);
     }
     IntVarArgs ma(1);
     ma[0] = m;
-    branch(this, ma, BVAR_NONE, BVAL_MIN);
-    if (opt.naive) {
-      branch(this, v, BVAR_SIZE_MIN, BVAL_MIN);
+    branch(this, ma, INT_VAR_NONE, INT_VAL_MIN);
+    if (opt.branching() == BRANCH_SIZE) {
+      branch(this, v, INT_VAR_SIZE_MIN, INT_VAL_MIN);
+    } else if (opt.branching() == BRANCH_DEGREE) {
+      branch(this, v, INT_VAR_DEGREE_MAX, INT_VAL_MIN);
     } else {
-      branch(this, v, BVAR_DEGREE_MAX, BVAL_MIN);
+      branch(this, v, INT_VAR_SIZE_DEGREE_MIN, INT_VAL_MIN);
     }
   }
   /// Constructor for cloning \a s
@@ -340,15 +368,15 @@ public:
   }
   /// Print the solution
   virtual void
-  print(void) {
-    std::cout << "\tm = " << m << std::endl
-	      << "\tv[] = {";
+  print(std::ostream& os) {
+    os << "\tm = " << m << std::endl
+       << "\tv[] = {";
     for (int i = 0; i < v.size(); i++) {
-      std::cout << v[i] << ", ";
+      os << v[i] << ", ";
       if ((i+1) % 15 == 0)
-	std::cout << std::endl << "\t       ";
+        os << std::endl << "\t       ";
     }
-    std::cout << "};" << std::endl;
+    os << "};" << std::endl;
   }
 };
 
@@ -357,13 +385,21 @@ public:
  *  \relates GraphColor
  */
 int
-main(int argc, char** argv) {
-  Options opt("GraphColor");
-  opt.naive      = false;
-  opt.icl        = ICL_DOM;
-  opt.iterations = 20;
+main(int argc, char* argv[]) {
+  SizeOptions opt("GraphColor");
+  opt.icl(ICL_DOM);
+  opt.iterations(20);
+  opt.model(GraphColor::MODEL_NONE);
+  opt.model(GraphColor::MODEL_NONE, "none",
+            "no lower bound");
+  opt.model(GraphColor::MODEL_CLIQUE, "clique",
+            "use maximal clique size as lower bound");
+  opt.branching(GraphColor::BRANCH_DEGREE);
+  opt.branching(GraphColor::BRANCH_DEGREE, "degree");
+  opt.branching(GraphColor::BRANCH_SIZE, "size");
+  opt.branching(GraphColor::BRANCH_SIZE_DEGREE, "sizedegree");
   opt.parse(argc,argv);
-  Example::run<GraphColor,DFS>(opt);
+  Example::run<GraphColor,DFS,SizeOptions>(opt);
   return 0;
 }
 
