@@ -1,3 +1,4 @@
+/* -*- mode: C++; c-basic-offset: 2; indent-tabs-mode: nil -*- */
 /*
  *  Main authors:
  *     Christian Schulte <schulte@gecode.org>
@@ -6,16 +7,31 @@
  *     Christian Schulte, 2003
  *
  *  Last modified:
- *     $Date: 2006-08-04 16:06:52 +0200 (Fri, 04 Aug 2006) $ by $Author: schulte $
- *     $Revision: 3517 $
+ *     $Date: 2007-11-30 13:58:34 +0100 (Fri, 30 Nov 2007) $ by $Author: tack $
+ *     $Revision: 5524 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
  *     http://www.gecode.org
  *
- *  See the file "LICENSE" for information on usage and
- *  redistribution of this file, and for a
- *     DISCLAIMER OF ALL WARRANTIES.
+ *  Permission is hereby granted, free of charge, to any person obtaining
+ *  a copy of this software and associated documentation files (the
+ *  "Software"), to deal in the Software without restriction, including
+ *  without limitation the rights to use, copy, modify, merge, publish,
+ *  distribute, sublicense, and/or sell copies of the Software, and to
+ *  permit persons to whom the Software is furnished to do so, subject to
+ *  the following conditions:
+ *
+ *  The above copyright notice and this permission notice shall be
+ *  included in all copies or substantial portions of the Software.
+ *
+ *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ *  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+ *  LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+ *  OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+ *  WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  */
 
@@ -24,7 +40,7 @@
 /**
  * \brief %Example: partition numbers into two groups
  *
- * \ingroup Example
+ * \ingroup ExProblem
  */
 class Partition : public Example {
 protected:
@@ -34,22 +50,21 @@ protected:
   IntVarArray y;
 public:
   /// Actual model
-  Partition(const Options& opt)
-    : x(this,opt.size,1,2*opt.size),
-      y(this,opt.size,1,2*opt.size) {
-    const int n = opt.size;
+  Partition(const SizeOptions& opt)
+    : x(this,opt.size(),1,2*opt.size()),
+      y(this,opt.size(),1,2*opt.size()) {
+    const int n = opt.size();
     // Break symmetries by ordering numbers in each group
-    for (int i = n-1; i--; ) {
-      rel(this, x[i], IRT_LE, x[i+1]);
-      rel(this, y[i], IRT_LE, y[i+1]);
-    }
+    rel(this, x, IRT_LE);
+    rel(this, y, IRT_LE);
+
     rel(this, x[0], IRT_LE, y[0]);
 
     IntVarArgs xy(2*n);
     for (int i = n; i--; ) {
       xy[i] = x[i]; xy[n+i] = y[i];
     }
-    distinct(this, xy, opt.icl);
+    distinct(this, xy, opt.icl());
     IntArgs c(2*n);
     for (int i = n; i--; ) {
       c[i] = 1; c[n+i] = -1;
@@ -72,7 +87,7 @@ public:
     linear(this, y, IRT_EQ, 2*n*(2*n+1)/4);
     linear(this, sx, IRT_EQ, 2*n*(2*n+1)*(4*n+1)/12);
     linear(this, sy, IRT_EQ, 2*n*(2*n+1)*(4*n+1)/12);
-    branch(this, xy, BVAR_SIZE_MIN, BVAL_MIN);
+    branch(this, xy, INT_VAR_SIZE_MIN, INT_VAL_MIN);
   }
 
   /// Constructor used during cloning \a s
@@ -87,23 +102,23 @@ public:
   }
   /// Print solution
   virtual void
-  print(void) {
-    std::cout << "\t";
+  print(std::ostream& os) {
+    os << "\t";
     int a, b;
     a = b = 0;
     for (int i = 0; i < x.size(); i++) {
       a += x[i].val();
       b += x[i].val()*x[i].val();
-      std::cout << x[i] << ", ";
+      os << x[i] << ", ";
     }
-    std::cout << " = " << a << ", " << b << std::endl << "\t";
+    os << " = " << a << ", " << b << std::endl << "\t";
     a = b = 0;
     for (int i = 0; i < y.size(); i++) {
       a += y[i].val();
       b += y[i].val()*y[i].val();
-      std::cout << y[i] << ", ";
+      os << y[i] << ", ";
     }
-    std::cout << " = " << a << ", " << b << std::endl;
+    os << " = " << a << ", " << b << std::endl;
   }
 };
 
@@ -112,12 +127,12 @@ public:
  * \relates Partition
  */
 int
-main(int argc, char** argv) {
-  Options o("Partition");
-  o.size = 32;
-  o.icl  = ICL_DOM;
-  o.parse(argc,argv);
-  Example::run<Partition,DFS>(o);
+main(int argc, char* argv[]) {
+  SizeOptions opt("Partition");
+  opt.size(32);
+  opt.icl(ICL_DOM);
+  opt.parse(argc,argv);
+  Example::run<Partition,DFS,SizeOptions>(opt);
   return 0;
 }
 
